<?php

namespace Drupal\footable\Plugin\views\style;

use Drupal\Core\Form\FormStateInterface;
use Drupal\footable\Entity\FooTableBreakpoint;
use Drupal\views\Plugin\views\style\Table;

/**
 * Style plugin to render a table as a FooTable.
 *
 * @ingroup views_style_plugins
 *
 * @ViewsStyle(
 *   id = "footable",
 *   title = @Translation("FooTable"),
 *   help = @Translation("Render a table as a FooTable."),
 *   theme = "views_view_footable",
 *   display_types = { "normal" }
 * )
 */
class FooTable extends Table {

  /**
   * {@inheritdoc}
   */
  protected $usesFields = TRUE;

  /**
   * {@inheritdoc}
   */
  protected $usesRowPlugin = FALSE;

  /**
   * {@inheritdoc}
   */
  protected $usesRowClass = TRUE;

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['footable'] = [
      'default' => [
        'expand_first' => FALSE,
        'show_header' => TRUE,
        'toggle_column' => 'first',
        'bootstrap' => [
          'striped' => FALSE,
          'bordered' => FALSE,
          'hover' => FALSE,
          'condensed' => FALSE,
        ],
      ],
    ];
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['footable'] = [
      '#type' => 'details',
      '#title' => $this->t('FooTable settings'),
      '#open' => TRUE,
    ];

    $form['footable']['expand_first'] = [
      '#title' => $this->t('Expand first row'),
      '#description' => $this->t('Whether or not to expand the first rows details.'),
      '#type' => 'select',
      '#options' => [
        FALSE => $this->t('Disabled'),
        TRUE => $this->t('Enabled'),
      ],
      '#default_value' => $this->options['footable']['expand_first'],
    ];

    $form['footable']['show_header'] = [
      '#title' => $this->t('Show header'),
      '#description' => $this->t('Whether or not to display a header row in the table.'),
      '#type' => 'select',
      '#options' => [
        TRUE => $this->t('Yes'),
        FALSE => $this->t('No'),
      ],
      '#default_value' => $this->options['footable']['show_header'],
    ];

    $form['footable']['toggle_column'] = [
      '#title' => $this->t('Expandable column'),
      '#description' => $this->t('Specify which column the toggle is appended to in a row.'),
      '#type' => 'select',
      '#options' => [
        'first' => $this->t('First'),
        'last' => $this->t('Last'),
      ],
      '#default_value' => $this->options['footable']['toggle_column'],
    ];

    // Bootstrap style configuration.
    if (\Drupal::config('footable.settings')->get('footable_plugin_type') == 'bootstrap') {
      $form['footable']['bootstrap'] = [
        '#type' => 'details',
        '#title' => $this->t('Bootstrap'),
      ];

      $form['footable']['bootstrap']['striped'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Striped'),
        '#default_value' => $this->options['footable']['bootstrap']['striped'],
      ];

      $form['footable']['bootstrap']['bordered'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Bordered'),
        '#default_value' => $this->options['footable']['bootstrap']['bordered'],
      ];

      $form['footable']['bootstrap']['hover'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Hover'),
        '#default_value' => $this->options['footable']['bootstrap']['hover'],
      ];

      $form['footable']['bootstrap']['condensed'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Condensed'),
        '#default_value' => $this->options['footable']['bootstrap']['condensed'],
      ];
    }

    // Breakpoint configuration.
    $form['footable']['breakpoint'] = [
      '#type' => 'details',
      '#title' => $this->t('Collapsed columns'),
      '#description' => $this->t('Select the "breakpoints" where a particular column should be hidden.'),
      '#open' => TRUE,
    ];

    $breakpoints = [];
    foreach (FooTableBreakpoint::loadAll() as $breakpoint) {
      $breakpoints[$breakpoint->id()] = $breakpoint->label();
    }

    if (!empty($breakpoints)) {
      foreach ($this->displayHandler->getFieldLabels() as $name => $label) {
        $form['footable']['breakpoint'][$name] = [
          '#title' => $label,
          '#type' => 'checkboxes',
          '#options' => $breakpoints,
          '#default_value' => isset($this->options['footable']['breakpoint'][$name]) ? $this->options['footable']['breakpoint'][$name] : NULL,
          '#multiple' => TRUE,
        ];
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validateOptionsForm(&$form, FormStateInterface $form_state) {
    parent::validateOptionsForm($form, $form_state);

    $breakpoints = FALSE;
    foreach ($form_state->getValue(array('style_options', 'footable', 'breakpoint'), array()) as $breakpoint) {
      if (!empty(array_filter($breakpoint))) {
        $breakpoints = TRUE;
        break;
      }
    }

    if (!$breakpoints) {
      $form_state->setErrorByName('style_options][footable][breakpoint', $this->t('You need to have atleast one column that has a breakpoint.'));
    }
  }

}